from typing import List, Tuple

from models.GameObject import GameObject
from models.enums.EnumGameObjectType import EnumGameObjectType


class CollectionEnemyTanks:
    def __init__(self, game_objects: List[GameObject]):
        super().__init__()
        self._enemy_tanks = [
            it for it in game_objects if
            it.game_object_type in [EnumGameObjectType.Enemy, EnumGameObjectType.EnemyAdvanced]
        ]

        self._idx = 0

        # self._enemy_tanks = []
        # for it in game_objects:
        #     if it.game_object_type in [EnumGameObjectType.Enemy, EnumGameObjectType.EnemyAdvanced]:
        #         self._enemy_tanks.append(it)

    # reset() called at any for loop in begining
    def __iter__(self):
        self._idx = 0
        return self

    # on each for loop call
    def __next__(self) -> GameObject:
        if self._idx >= len(self._enemy_tanks):
            raise StopIteration() # exit for loop
        enemy_tank = self._enemy_tanks[self._idx] # if I want to return group [self._idx:self._idx+size]
        self._idx += 1
        return enemy_tank


if __name__ == "__main__":
    game_objects = []
    game_objects.append(GameObject())
    game_objects[-1].game_object_type = EnumGameObjectType.Forest
    game_objects.append(GameObject())
    game_objects[-1].game_object_type = EnumGameObjectType.Enemy
    game_objects.append(GameObject())
    game_objects[-1].game_object_type = EnumGameObjectType.Enemy

    dummy_collection = CollectionEnemyTanks(game_objects)
    for pos, tank in dummy_collection:
        print(pos, tank.game_object_type)