import os.path
import time
from dataclasses import dataclass, field
from typing import List

import pygame

from models.HighScore import HighScore
from models.HighScores import HighScores
from views.components.ReactButton import ReactButton, ReactPropsButton
from views.components.ReactTextBox import ReactTextBox, ReactPropsTextBox
from views.components.ReactTextField import ReactTextField, ReactPropsTextField
from views.events.EventKey import EventKey
from views.events.EventMouse import EventMouse
from views.react.ReactComponent import ReactProps, ReactComponent, ReactState


@dataclass
class ReactPropsWindowHighScores(ReactProps):
    current_score: int = 0


@dataclass
class ReactStateWindowHighScores(ReactState):
    high_scores: HighScores = None
    high_scores_filename: str = "high_scores.json"
    current_message: str = ""
    is_entering: bool = False

#TODO button to close window

class WindowHighScores(ReactComponent):
    def __init__(self, props: ReactPropsWindowHighScores):
        super().__init__(props)
        self.props = props

        self.state = ReactStateWindowHighScores(
            high_scores= HighScores()
        )

        #TODO

    def onPressSave(self):
        #TODO

        with open(self.state.high_scores_filename, "w") as file:
            file.write(self.state.high_scores.to_json())

    def onKeyDown(self, event: EventKey):
        super().onKeyDown(event)

    def onTextChange(self, text):
        #TODO
        pass

    def render(self):
        contents: List[ReactComponent] = []

        # TODO
        return contents

    def draw(self, screen):
        if self.shouldComponentUpdate(self.props, self.state):
            pygame.draw.rect(
                screen,
                color=(255, 255, 255),
                rect=pygame.Rect(
                    self.props.x,
                    self.props.y,
                    self.props.width,
                    self.props.height
                )
            )
        super().draw(screen)


    def show(self, screen):
        is_running = True
        is_mouse_down = False
        while is_running:
            mouse_pos = pygame.mouse.get_pos()
            mouse_buttons = pygame.mouse.get_pressed()

            if is_mouse_down and not any(mouse_buttons):
                event = EventMouse()
                event.x = mouse_pos[0]
                event.y = mouse_pos[1]
                event.is_mouse_down = False
                self.onPress(event)
            else:
                event = EventMouse()
                event.x = mouse_pos[0]
                event.y = mouse_pos[1]
                event.is_mouse_down = any(mouse_buttons)
                self.onMouseMove(event)
            is_mouse_down = any(mouse_buttons)

            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    is_running = False
                elif event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_ESCAPE:
                        is_running = False
                    else:
                        event = EventKey(
                            key_code=event.key,
                            key_name=pygame.key.name(event.key)
                        )
                        self.onKeyDown(event)

            self.draw(screen)
            pygame.display.flip()
            time.sleep(0.01)